<?php

declare(strict_types=1);

namespace mirzaev\parser_from_rossko;

use avadim\FastExcelReader\Excel as excel;

use ZipArchive as zip,
	DirectoryIterator as directory;

use SQLite3 as db;

use GuzzleHttp\Client as client,
	GuzzleHttp\Psr7\Request as request,
	GuzzleHttp\Psr7\Utils as utils;

use DateTimeImmutable as datetimeimmutable;

ini_set('error_reporting', E_ALL);
ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);

define('VIEWS', realpath(__DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'views'));
define('STORAGE', realpath(__DIR__ . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'storage'));
define('INDEX', __DIR__);

// Автозагрузка
require INDEX . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . '..' . DIRECTORY_SEPARATOR . 'vendor' . DIRECTORY_SEPARATOR . 'autoload.php';

// Инициализация инстанции абстрактной базы данных
$db = new db(STORAGE . DIRECTORY_SEPARATOR . 'products.db');

// Инициализация таблицы
$db->exec('CREATE TABLE IF NOT EXISTS product (id INTEGER PRIMARY KEY AUTOINCREMENT, avito INTEGER UNIQUE, nomenclature TEXT, brand TEXT, code TEXT, description TEXT, weight INTEGER, multiplicity REAL, cost_rossko REAL, cost REAL, amount TEXT, delivery INTEGER, code_rossko TEXT UNIQUE, oem TEXT, applicability TEXT, code_vendor TEXT, warehouse TEXT, updated DATETIME DEFAULT CURRENT_TIMESTAMP);');

// Инициализация директории с документом Rossko
define('ROSSKO', STORAGE . DIRECTORY_SEPARATOR . 'rossko');

// Инициализация хранилища фотографий товаров
define('IMAGES', STORAGE . DIRECTORY_SEPARATOR . 'images');

// Объявление документа
$document;

foreach (new directory(ROSSKO) as $file)
	if ($file->getExtension() === 'xlsx') {
		// Инициализация документа
		$document = ROSSKO . DIRECTORY_SEPARATOR . $file->getFilename();
	};

if (empty($document)) {
	// Инициализация архива
	$archive = ROSSKO . DIRECTORY_SEPARATOR . 'rossko_price.zip';

	if (($zip = new zip)->open($archive)) {
		// Открыт архив

		// Разархивирование документа
		$zip->extractTo(ROSSKO);
		$zip->close();

		// Удаление архива (необязательно)
		unlink($archive);
	}

	foreach (new directory(ROSSKO) as $file)
		if ($file->getExtension() === 'xlsx') {
			// Инициализация документа
			$document = ROSSKO . DIRECTORY_SEPARATOR . $file->getFilename();
		};
}

if (!empty($document) ?? file_exists($document)) {
	// Инициализирован документ

	/**
	 * Скачать изображение товара
	 */
	function image(string $nomenclature)
	{
		// Если существуют файлы (подразумеваются изображения), то пропустить проверку и скачивание изображений
		if (file_exists($image = IMAGES . DIRECTORY_SEPARATOR . $nomenclature))
			foreach (new directory($image) as $file) return;

		// Инициализация клиента Guzzle
		$client = new client(['cookies' => true]);

		// Инициализация сессии (cookie)
		/* $cookies = ($client->send(new request('GET', 'https://krsk.rossko.ru/')))->getHeader('Set-Cookie');
		$_auth = '';
		foreach ($cookies as $cookie)
			if (str_starts_with($cookie, 'auth')) {
				preg_match('/auth=(.*);/', $cookie, $matches);
				$_auth = $matches[1];
			}
		if (empty($_auth)) return false; */

		// Инициализация запроса
		/* $request = new request(
			'GET',
			"https://productcard.rossko.ru/api/Product/Card/$nomenclature",
			[ */
		// 'Accept' => '*/*',
		/* 'Accept-Encoding' => 'gzip, deflate, br, zstd',
				'Accept-Language' => 'ru-RU,ru;q=0.8,en-US;q=0.5,en;q=0.3',
				'Access-Control-Allow-Origin' => '*',
				'Authorization-Domain' => 'https://krsk.rossko.ru',
				'Authorization-Session' => $_auth,
				'Cache-Control' => 'no-cache',
				'Connection' => 'keep-alive',
				'Content-Type' => 'application/json',
				'DNT' => '1',
				'Host' => 'productcard.rossko.ru',
				'Origin' => 'https://krsk.rossko.ru',
				'Pragma' => 'no-cache',
				'Priority' => 'u=4',
				'Referer' => 'https://krsk.rossko.ru/',
				'Sec-Fetch-Dest' => 'empty',
				'Sec-Fetch-Mode' => 'no-cors',
				'Sec-Fetch-Site' => 'same-site',
				'Sec-GPC' => '1',
				'source' => 'frontend',
				'TE' => 'trailers',
				'User-Agent' => 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:129.0) Gecko/20100101 Firefox/129.0',
			]
		);	 */

		// ОНИ НЕ ПРОВЕРЯЮТ ПОДЛИННОСТЬ СЕССИИ, ХА
		// Запрос данных карточки товара в разделе "Описание"
		$request = new request(
			'GET',
			"https://productcard.rossko.ru/api/Product/Card/$nomenclature",
			[
				'Authorization-Domain' => 'https://krsk.rossko.ru',
				'Authorization-Session' => 'ya_tvoi_kartinki_spizdil',
			]
		);

		// Отправка запроса и инициализация ответа
		$response = $client->send($request);

		// Инициализация изображений
		$images = json_decode((string) $response->getBody(), true)['mainPart']['images'] ?? null;

		foreach ($images ?? [] as $image) {
			// Перебор изображений

			// Инициализация пути до директории изображения 
			$path_directory = IMAGES . DIRECTORY_SEPARATOR . $nomenclature;

			// Инициализация пути до изображения
			$path_image = $path_directory . DIRECTORY_SEPARATOR . preg_replace('/.*\//', '', $image);

			// Инициализация директории изображения
			if (!file_exists($path_directory)) mkdir($path_directory, 0775, true);

			/* var_dump("Проверка: $path_image"); */

			// Инициализация изображения (скачивание)
			if (!file_exists($path_image)) {
				/* var_dump("Загрузка: $path_image"); */
				$client->request('GET', $image, ['sink' => $path_image]);
			}
		}
	}

	/**
	 * Скачать список автомобилей для которых продукт подходит
	 */
	function applicability(string $nomenclature)
	{
		// Инициализация клиента Guzzle
		$client = new client(['cookies' => true]);

		// Запрос данных карточки товара в разделе "Применимость"
		$request = new request(
			'GET',
			"https://productcard.rossko.ru/api/Product/CarTypes/$nomenclature",
			[
				'Authorization-Domain' => 'https://krsk.rossko.ru',
				'Authorization-Session' => 'ya_tvoi_applicabilities_spizdil',
			]
		);

		// Отправка запроса и инициализация ответа
		$response = $client->send($request);

		// Инициализация данных автомобилей и возврат (успех)
		return (string) $response->getBody();
	}

	// Инициализация таблицы
	$spreadsheet = excel::open($document);

	// Инициализация рабочего листа
	$worksheet = $spreadsheet->sheet();

	foreach (
		$worksheet->nextRow(
			[
				'A' => 'nomenclature',
				'B' => 'brand',
				'C' => 'code',
				'D' => 'description',
				'E' => 'weight',
				'F' => 'multiplicity',
				'G' => 'cost_rossko',
				'H' => 'cost',
				'I' => 'amount',
				'J' => 'delivery',
				'K' => 'code_rossko',
				'L' => 'oem',
				'M' => 'applicability',
				'N' => 'code_vendor',
				'O' => 'warehouse'
			],
			excel::KEYS_FIRST_ROW
		) as $number => $row
	) {
		// Перебор строк

		if (!empty($row['nomenclature'])) {
			// Инициализирован "nomenclature" (номенклатура)

			// Инициализация запроса: "SELECT" (по номенклатуре получить applicability и updated)
			$request = $db->prepare('SELECT applicability, updated FROM product WHERE nomenclature=?;');

			// Инициализация параметров запроса
			$request->bindValue(1, $row['nomenclature'] ?? null, SQLITE3_TEXT);

			// Выполнение запроса
			$response = $request->execute()?->fetchArray();

			if (
				empty($response['updated'])
				|| (!empty($_current = datetimeimmutable::createFromFormat(DATE_W3C, date(DATE_W3C)))
				&& !empty($_updated = datetimeimmutable::createFromFormat(DATE_W3C, $response['updated']))
				&& $_current->diff($_updated)->h > 24)
			) {
				// Записи не существует (может быть не инициализирована) ИЛИ запись обновлялась более чем 24 часа назад
				
				// Объявление переменной для значения "applicability" ()применимость)
				$applicability;

				if (empty($response['applicability'])) {
					// Не найдено значение: "applicability" (применимость)

					// Инициализация значения: "applicability" (применимость)
					$applicability = applicability($row['nomenclature']);
				}

				// Инициализация запроса: "REPLACE" (INSERT или UPDATE)
				$request = $db->prepare('REPLACE INTO product (nomenclature, brand, code, description, weight, multiplicity, cost_rossko, cost, amount, delivery, code_rossko, oem, applicability, code_vendor, warehouse, updated) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);');

				// Инициализация параметров запроса
				$request->bindValue(1, $row['nomenclature'] ?? null, SQLITE3_TEXT);
				$request->bindValue(2, $row['brand'] ?? null, SQLITE3_TEXT);
				$request->bindValue(3, $row['code'], SQLITE3_TEXT);
				$request->bindValue(4, $row['description'] ?? null, SQLITE3_TEXT);
				$request->bindValue(5, $row['weight'] ?? null, SQLITE3_INTEGER);
				$request->bindValue(6, $row['multiplicity'] ?? null, SQLITE3_FLOAT);
				$request->bindValue(7, $row['cost_rossko'] ?? null, SQLITE3_FLOAT);
				$request->bindValue(8, $row['cost'] ?? null, SQLITE3_FLOAT);
				$request->bindValue(9, $row['amount'] ?? null, SQLITE3_TEXT);
				$request->bindValue(10, $row['delivery'] ?? null, SQLITE3_INTEGER);
				$request->bindValue(11, $row['code_rossko'] ?? null, SQLITE3_TEXT);
				$request->bindValue(12, $row['oem'] ?? null, SQLITE3_TEXT);
				$request->bindValue(13, $applicability ?? null, SQLITE3_TEXT);
				$request->bindValue(14, $row['code_vendor'] ?? null, SQLITE3_TEXT);
				$request->bindValue(15, $row['warehouse'] ?? null, SQLITE3_TEXT);
				$request->bindValue(16, date(DATE_W3C), SQLITE3_TEXT);

				// Выполнение запроса
				$request->execute();

				if (!file_exists(IMAGES . DIRECTORY_SEPARATOR . $row['nomenclature'] . '1.jpg')) {
					// Не найдено изображение товара

					// Ожидание перед загрузкой изображения
					/* sleep(10); */

					// Загрузка изображения товара
					image($row['nomenclature']);
				}
			}
		}
	}

	// Удаление документа
	unlink($document);
}
